\version "2.24.1"

#(begin
  (define debug #f)

  (define (simplebar bartype)
    (or (string=? bartype "")
        (string=? bartype "|")
        (string=? bartype ".")
        (string=? bartype ";")
        (string=? bartype "!")
        (string=? bartype "'")))

  (define (specialbar bartype barleft)
    (or (string=? bartype ".|:-||")
        (string=? barleft "||")))

  ; modify line-break-penalty for bars in barlist
  (define (Break_engraver barlist weight)
    (lambda (score)
      (make-engraver

        (acknowledgers
          ((paper-column-interface engraver grob source-engraver)
            (let* ((barnum  (ly:context-property score 'currentBarNumber 0))
                   (barline (ly:context-property score 'currentBarLine #f))
                   (bartype (if barline (ly:grob-property barline 'glyph) ""))
                   (barleft (if barline (ly:grob-property barline 'glyph-left) ""))
                   (simple  (simplebar bartype))
                   (special (specialbar bartype barleft))
                   (scale   (if simple 1.0 (if special -1.5 -1.0))))
              (if (memq barnum barlist)
                (ly:grob-set-property! grob 'line-break-penalty (* scale weight)))))

          ((bar-line-interface engraver grob source-engraver)
            (let* ((barnum  (ly:context-property score 'currentBarNumber 0))
                   (barline (ly:context-property score 'currentBarLine #f))
                   (bartype (if barline (ly:grob-property barline 'glyph) ""))
                   (barleft (if barline (ly:grob-property barline 'glyph-left) ""))
                   (simple  (simplebar bartype))
                   (special (specialbar bartype barleft))
                   (colour  (if simple red (if special blue green))))
              (and debug
                   (memq barnum barlist)
                   (ly:grob-set-property! grob 'color colour))))
        ))))

  ; find bars where line-break-penalty should be adjusted
  (define (Bar_List_engraver)
    (lambda (score)
      (let ((zero    (ly:make-moment 0 0 0 0))
            (global  (ly:context-parent score))
            (barlist '()))

      (make-engraver
        ((finalize translator)
          (ly:context-set-property! global 'barlist barlist))

        (acknowledgers
          ((bar-line-interface engraver grob source-engraver)
            (let* ((barnum   (ly:context-property score 'currentBarNumber 0))
                   (barline  (ly:context-property score 'currentBarLine #f))
                   (bartype  (if barline (ly:grob-property barline 'glyph) "|"))
                   (position (ly:context-property score 'measurePosition 0)))
              (and (not (simplebar bartype))
                   (not (memq barnum barlist))
                   (set! barlist (append barlist (list (1- barnum) barnum (1+ barnum))))
                   (ly:moment<? zero position)
                   (set! barlist (append barlist (list (+ 2 barnum)))))))
        )))))
)
